Package: spatemR
Type: Package
Title: Generalized Spatial Autoregresive Models for Mean and Variance
Version: 1.3.0
Authors@R: c(person("Nelson Alirio", "Cruz Gutierrez", 
                role = c("aut", "cre","cph"), email = "nelson-alirio.cruz@uib.es"), 
          person("Oscar Orlando", "Melo", role = "aut",
                email="oomelom@unal.edu.co"),
          person("Jurgen", "Toloza-Delgado", role = "aut",
                email="jdtolozad@unal.edu.co"))
Description: Modeling spatial dependencies in dependent variables, extending traditional spatial regression approaches. It allows for the joint modeling of both the mean and the variance of the dependent variable, incorporating semiparametric effects in both models. Based on generalized additive models (GAM), the package enables the inclusion of non-parametric terms while maintaining the classical theoretical framework of spatial regression. Additionally, it implements the Generalized Spatial Autoregression (GSAR) model, which extends classical methods like logistic Spatial Autoregresive Models (SAR), probit Spatial Autoregresive Models (SAR), and Poisson Spatial Autoregresive Models (SAR), offering greater flexibility in modeling spatial dependencies and significantly improving computational efficiency and the statistical properties of the estimators. Related work includes: a) J.D. Toloza-Delgado, Melo O.O., Cruz N.A. (2024). "Joint spatial modeling of mean and non-homogeneous variance combining semiparametric SAR and GAMLSS models for hedonic prices". <doi:10.1016/j.spasta.2024.100864>. b) Cruz, N. A., Toloza-Delgado, J. D., Melo, O. O. (2024). "Generalized spatial autoregressive model". <doi:10.48550/arXiv.2412.00945>. 
License: GPL (>= 3)
Encoding: UTF-8
Depends: R (>= 4.0)
Imports: methods, stats, gamlss (>= 5.3-1), gamlss.dist, sphet, Matrix
Suggests: testthat (>= 3.0.0), sp, spdep
Config/testthat/edition: 3
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-14 17:49:17 UTC; UIB
Author: Nelson Alirio Cruz Gutierrez [aut, cre, cph],
  Oscar Orlando Melo [aut],
  Jurgen Toloza-Delgado [aut]
Maintainer: Nelson Alirio Cruz Gutierrez <nelson-alirio.cruz@uib.es>
Repository: CRAN
Date/Publication: 2026-04-14 18:00:09 UTC
Built: R 4.4.3; ; 2026-04-14 19:11:34 UTC; unix

Package: spatemR
Type: Package
Title: Generalized Spatial Autoregresive Models for Mean and Variance
Version: 1.3.0
Authors@R: c(person("Nelson Alirio", "Cruz Gutierrez", 
                role = c("aut", "cre","cph"), email = "nelson-alirio.cruz@uib.es"), 
          person("Oscar Orlando", "Melo", role = "aut",
                email="oomelom@unal.edu.co"),
          person("Jurgen", "Toloza-Delgado", role = "aut",
                email="jdtolozad@unal.edu.co"))
Description: Modeling spatial dependencies in dependent variables, extending traditional spatial regression approaches. It allows for the joint modeling of both the mean and the variance of the dependent variable, incorporating semiparametric effects in both models. Based on generalized additive models (GAM), the package enables the inclusion of non-parametric terms while maintaining the classical theoretical framework of spatial regression. Additionally, it implements the Generalized Spatial Autoregression (GSAR) model, which extends classical methods like logistic Spatial Autoregresive Models (SAR), probit Spatial Autoregresive Models (SAR), and Poisson Spatial Autoregresive Models (SAR), offering greater flexibility in modeling spatial dependencies and significantly improving computational efficiency and the statistical properties of the estimators. Related work includes: a) J.D. Toloza-Delgado, Melo O.O., Cruz N.A. (2024). "Joint spatial modeling of mean and non-homogeneous variance combining semiparametric SAR and GAMLSS models for hedonic prices". <doi:10.1016/j.spasta.2024.100864>. b) Cruz, N. A., Toloza-Delgado, J. D., Melo, O. O. (2024). "Generalized spatial autoregressive model". <doi:10.48550/arXiv.2412.00945>. 
License: GPL (>= 3)
Encoding: UTF-8
Depends: R (>= 4.0)
Imports: methods, stats, gamlss (>= 5.3-1), gamlss.dist, sphet, Matrix
Suggests: testthat (>= 3.0.0), sp, spdep
Config/testthat/edition: 3
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-14 17:49:17 UTC; UIB
Author: Nelson Alirio Cruz Gutierrez [aut, cre, cph],
  Oscar Orlando Melo [aut],
  Jurgen Toloza-Delgado [aut]
Maintainer: Nelson Alirio Cruz Gutierrez <nelson-alirio.cruz@uib.es>
Repository: CRAN
Date/Publication: 2026-04-14 18:00:09 UTC
Built: R 4.6.0; ; 2026-04-28 11:28:02 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,summary.GSCIMC)
S3method(print,summary.SARARgamlss)
export(GSCIMC)
export(Hurdle_GSCIMC)
export(SARARgamlss)
export(ptfamily)
export(summary_SAR)
export(var_rho_inv)
import(gamlss)
import(splines)
importFrom(gamlss.dist,NO)
importFrom(methods,is)
importFrom(sphet,spreg)
importFrom(stats,.getXlevels)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,dpois)
importFrom(stats,fitted)
importFrom(stats,gaussian)
importFrom(stats,glm.fit)
importFrom(stats,make.link)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.offset)
importFrom(stats,model.response)
importFrom(stats,optim)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,rpois)
importFrom(stats,update)

# spatemR
An R package for autoregresive spatial models with autoregresive temporal trend and GAMLSS

Use `devtools::install_github("Cruzalirio/spatemR")` to install the latest version
of the package

#' Truncated Poisson Family for GLM
#'
#' This function defines a truncated Poisson family for use in Generalized Linear Models (GLMs), 
#' where zero values are not allowed. It modifies the Poisson likelihood by excluding zero-count observations.
#'
#' @param link Character string or a link-glm object specifying the link function. Accepted values are "log", "identity", and "sqrt".
#' @return An object of class "family" that can be used in \code{glm()}.
#'
#' @examples
#' set.seed(123)
#' y <- rpois(100, lambda = 3)
#' y <- y[y > 0]  # Truncate zeros
#' x <- rnorm(length(y))
#' model <- glm(y ~ x, family = ptfamily())
#' summary(model)
#'
#'@export

ptfamily <- function (link = "log") 
{
  linktemp <- substitute(link)
  if (!is.character(linktemp)) 
    linktemp <- deparse(linktemp)
  okLinks <- c("log", "identity", "sqrt")
  family <- "ptfamily"
  if (linktemp %in% okLinks) 
    stats <- make.link(linktemp)
  else if (is.character(link)) {
    stats <- make.link(link)
    linktemp <- link
  }
  else {
    if (inherits(link, "link-glm")) {
      stats <- link
      if (!is.null(stats$name)) 
        linktemp <- stats$name
    }
    else {
      stop(gettextf("link \"%s\" not available for %s family; available links are %s", 
                    linktemp, family, paste(sQuote(okLinks), collapse = ", ")), 
           domain = NA)
    }
  }
  variance <- function(mu) mu * (1 - exp(-mu)) / (1 - exp(-mu) - mu * exp(-mu))
  validmu <- function(mu) all(is.finite(mu)) && all(mu > 0)
  dev.resids <- function(y, mu, wt) {
    -2 * wt * (y * log(mu) - mu - lfactorial(y) - log(1 - exp(-mu)))
  }
  aic <- function(y, n, mu, wt, dev) -2 * sum(dpois(y, lambda = mu, log = TRUE) - log(1 - exp(-mu)))
  initialize <- expression({
    if (any(y < 1)) stop("Values less than 1 not allowed for the 'Poisson truncated' family")
    n <- rep.int(1, nobs)
    mustart <- y + 0.1
  })
  simfun <- function(object, nsim) {
    wts <- object$prior.weights
    if (any(wts != 1)) 
      warning("ignoring prior weights")
    ftd <- fitted(object)
    rpois(nsim * length(ftd), ftd)
  }
  structure(list(family = family, link = linktemp, linkfun = stats$linkfun, 
                 linkinv = stats$linkinv, variance = variance, dev.resids = dev.resids, 
                 aic = aic, mu.eta = stats$mu.eta, initialize = initialize, 
                 validmu = validmu, valideta = stats$valideta, simulate = simfun, 
                 dispersion = 1), class = "family")
}

#' Hurdle Model using GSCIMC
#'
#' This function fits a hurdle model using GSCIMC, consisting of:
#' (1) A logit model for zero vs. non-zero responses.
#' (2) A truncated Poisson model for positive counts.
#'
#' @param formula A formula specifying the model.
#' @param data The dataset.
#' @param W The spatial weight matrix.
#' @param weights Optional weights.
#' @param toler Convergence tolerance.
#' @param maxit Maximum number of iterations.
#' @param trace Logical. If TRUE, prints progress.
#' @return A list containing the logit and Poisson-truncated models.
#' 
#' @examples
#' \donttest{
#' set.seed(123)
#' n <- 100
#' x <- rnorm(n)
#' y <- rpois(n, lambda = exp(0.5 * x))
#' y[rbinom(n, 1, 1/(1+exp(-0.5*x)))] <- 0  # Introduce zeros
#' W <- matrix(rbinom(n^2,1,0.2), n, n)  # Example spatial weight matrix
#' diag(W) <- 0
#' rtot <- rowSums(W)
#' W <- W/ifelse(rtot==0, 0.1, rtot)
#' model <- Hurdle_GSCIMC(y ~ x, data = data.frame(y, x), W = W)
#' summary_SAR(model$logit_model)
#' summary_SAR(model$poisson_truncated_model)
#'}
#'
#'@export
Hurdle_GSCIMC <- function(formula, data, W, weights = NULL, toler = 1e-05, maxit = 200, trace = FALSE) {
  
  # Modelo logit para y == 0 vs y > 0
  data$y_binary <- as.numeric(data[[all.vars(formula)[1]]] > 0)
  
  GSCIMC_logit <- GSCIMC(
    formula = update(formula, y_binary ~ .),
    family = binomial(link = "logit"),
    data = data,
    W = W,
    weights = weights,
    toler = toler,
    maxit = maxit,
    trace = trace
  )
  
  # Modelo Poisson truncado para y > 0
  data_positive <- subset(data, data[[all.vars(formula)[1]]] > 0)
  
  GSCIMC_poisson_truncated <- GSCIMC(
    formula = formula,
    family = ptfamily(),
    data = data_positive,
    W = W[data[[all.vars(formula)[1]]] > 0, data[[all.vars(formula)[1]]] > 0],
    weights = weights[data[[all.vars(formula)[1]]] > 0],
    toler = toler,
    maxit = maxit,
    trace = trace )
  
  return(list(logit_model = GSCIMC_logit, poisson_truncated_model = GSCIMC_poisson_truncated))
}

#' Generalized Estimating Equations with Spatial Autoregressive Components
#'
#' @description
#' `GSCIMC` estimates generalized estimating equations (GEE) incorporating spatial autoregressive (SAR) components.  
#' It extends GEE models to account for spatial dependence in the response variable.
#'
#' @param formula A formula specifying the model structure (response ~ predictors).
#' @param family A description of the error distribution and link function. Default is `gaussian()`.
#' @param weights Optional vector of prior weights. Must be positive.
#' @param data A data frame containing the variables in the model.
#' @param W A spatial weights matrix defining the spatial dependence structure.
#' @param start Optional starting values for parameter estimation.
#' @param toler Convergence tolerance for iterative optimization. Default is `1e-05`.
#' @param maxit Maximum number of iterations for model fitting. Default is `50`.
#' @param trace Logical; if `TRUE`, prints iteration details. Default is `FALSE`.
#' @param eps Minimun value for variance diference of zero
#'
#' @details
#' The function estimates a spatially autoregressive GEE model by iteratively updating the spatial dependence 
#' parameter (`rho`) and regression coefficients (`beta`). The estimation follows a quasi-likelihood approach 
#' using iterative weighted least squares (IWLS).  
#'
#' The function supports common GLM families (`gaussian`, `binomial`, `poisson`, `Gamma`, `inverse.gaussian`) and 
#' their quasi-likelihood equivalents.
#'
#' @return A list of class `"GSCIMC"` containing:
#' \item{coefficients}{Estimated regression coefficients.}
#' \item{rho}{Estimated spatial autoregressive parameter.}
#' \item{fitted.values}{Predicted values from the model.}
#' \item{linear.predictors}{Linear predictor values (`X * beta`).}
#' \item{prior.weights}{Weights used in estimation.}
#' \item{y}{Observed response values.}
#' \item{formula}{Model formula.}
#' \item{call}{Function call used to fit the model.}
#' \item{data}{Data used in the model.}
#' \item{converged}{Logical indicating whether the algorithm converged.}
#' \item{logLik}{Quasi-log-likelihood of the fitted model.}
#' \item{deviance}{Residual deviance.}
#' \item{df.residual}{Residual degrees of freedom.}
#' \item{phi}{Dispersion parameter estimate.}
#' \item{R}{Robust Variance Estimation.}
#' \item{CIC}{Corrected Information Criterion.}
#' \item{RJC}{Robust Jackknife Correction.}
#'
#' @seealso
#' \code{\link{glm}}, \code{\link[gee]{gee}}, \code{\link[spdep]{spdep}}
#'
#' @references Cruz, N. A., Toloza-Delgado, J. D., & Melo, O. O. (2024). 
#' Generalized spatial autoregressive model. arXiv preprint arXiv:2412.00945.
#' @source https://doi.org/10.48550/arXiv.2412.00945
#'
#' @examples
#' \donttest{
#' library(spdep)
#' library(sp)
#' data(meuse)
#' sp::coordinates(meuse) <- ~x+y
#' W <- spdep::nb2mat(knn2nb(knearneigh(meuse, k=5)), style="W")
#' fit <- GSCIMC(cadmium ~ dist + elev, family=poisson(), data=meuse, W=W)
#' summary_SAR(fit)
#'}
#' @export
#' @importFrom sphet spreg

GSCIMC <- function (formula, family = gaussian(), weights=NULL, data, W,
                    start = NULL, 
                    toler = 1e-04, maxit = 200, trace = FALSE, eps=1e-6) {
  mf <- stats::model.frame(formula, data=data)
  y <- base::as.matrix(model.response(mf))
  if (is(family, "function")) 
    family <- stats::family()
  if (family$family %in% c("quasi", "quasibinomial", "quasipoisson")) {
    if (family$family == "quasi") {
      family$family <- switch(family$varfun, constant = "gaussian", 
                              `mu(1-mu)` = "binomial", mu = "poisson", `mu^2` = "Gamma", 
                              `mu^3` = "inverse.gaussian")
    }
    else {
      family$family <- switch(family$family, quasibinomial = "binomial", 
                              quasipoisson = "poisson")
    }
    family <- do.call(family$family, list(link = family$link))
    family2 <- family
  }else {
    if (family$family %in% c("gaussian", "binomial", "poisson", 
                             "Gamma", "inverse.gaussian")) {
      varfun <- switch(family$family, gaussian = "constant", 
                       binomial = "mu(1-mu)", poisson = "mu", Gamma = "mu^2", 
                       inverse.gaussian = "mu^3")
      family2 <- do.call("quasi", list(variance = varfun, 
                                       link = family$link))
      if (family$family == "binomial") 
        family2 <- do.call("quasibinomial", list(link = family$link))
      if (family$family == "poisson") 
        family2 <- do.call("quasipoisson", list(link = family$link))
    }else family2 <- family
  }
  if (ncol(y) == 2 & family$family == "binomial") {
    weights <- as.matrix(y[, 1] + y[, 2])
    y <- as.matrix(y[, 1]/weights)
  }
  y <- as.matrix(as.numeric(y))
  offset <- as.vector(model.offset(mf))
  X <- model.matrix(formula, data)
  p <- ncol(X)
  n <- nrow(X)
  if (is.null(offset)){ 
    offs <- matrix(0, n, 1)}else{ offs <- as.matrix(offset)}
  if (is.null(weights)) {
    weights <- matrix(1, n, 1)} else {
      weights <- as.matrix(weights)}
  if (any(weights <= 0)) 
    stop("Only positive weights are allowed!!", call. = FALSE)
  datas <- cbind(X,y, offs, weights)
  
  
  qll <- NULL
  if (family$family == "gaussian") 
    qll <- function(weights, y, mu) -weights * (y - mu)^2/2
  if (family$family == "binomial") 
    qll <- qll <- function(weights, y, mu){
      eps <- eps
      mu <- pmin(pmax(mu, eps), 1-eps)
      weights * (y * log(mu) + (1 - y) * log(1 - mu))
    }
  if (family$family == "poisson") 
    qll <- function(weights, y, mu) weights * (y * log(mu) - mu)
  if (family$family == "Gamma") 
    qll <- function(weights, y, mu) -weights * (y/mu + log(mu))
  if (family$family == "inverse.gaussian") 
    qll <- function(weights, y, mu) weights * (mu - y/2)/mu^2
  if (family$family == "ptfamily") 
    qll <- function(weights, y, mu) weights * (y*log(mu)-mu-log(1-exp(-mu)))
  if (family$family == "Negative Binomial") {
    .Theta <- function() return(.Theta)
    environment(.Theta) <- environment(family$variance)
    .Theta <- .Theta()
    qll <- function(weights, y, mu) weights * (y * log(mu/(mu + .Theta)) + .Theta * 
                                                 log(.Theta/(.Theta + mu)))
  }
  if (family$family == "Tweedie") {
    .Theta <- function() return(p)
    environment(.Theta) <- environment(family$variance)
    .Theta <- .Theta()
    if (.Theta %in% c(0, 1, 2)) {
      if (.Theta == 0) 
        qll <- function(weights, y, mu) -weights * (y - mu)^2/2
      if (.Theta == 1) 
        qll <- function(weights, y, mu) weights * (y * log(mu) - mu)
      if (.Theta == 2) 
        qll <- function(weights, y, mu) -weights * (y/mu + log(mu))
    }
    else qll <- function(weights, y, mu) mu^(-.Theta) * (mu * y/(1 - .Theta) - mu^2/(2 - 
                                                                                       .Theta))
  }
  if (is.null(qll)) 
    qll <- function(weights, y, mu) family$dev.resids(y, mu, weights)
  
  qllp <- function(rho, beta, W, D,n){
    A <- Matrix::Diagonal(n) - rho*W
    Xi <-Matrix::solve(A + Matrix::Diagonal(n, eps),Matrix::Matrix(D[, 1:p]))
    yi <- D[, p + 1]
    ni <- nrow(Xi)
    etai <- Xi %*% beta + D[, p + 2]
    mui <- family$linkinv(etai[,1])
    if(family$family=="binomial"){
      mui <- pmin(pmax(mui,eps),1-eps)
    }
    QL <- sum(qll(weights=D[,p+3], y=D[,p+1], mu=mui))
    return(-QL)
  }
  
  if(family2$family=="ptfamily" &  !all(y>0)){
    stop("Only y>1 are allowed in ptfamily!!", call. = FALSE)
  }
  if (family$family=="poisson"){
  modTemp <- try(sphet::spreg(family$linkfun(y+1)~datas[, 1:p]-1,
                              listw = W, model = "lag"),
                 silent = TRUE)
  if(!inherits(modTemp, "try-error")){
    rho <- modTemp$coefficients[p+1]  
  }else{
    rho <- 0
  }}else if(family$family=="binomial"){
    glm_temp <- glm.fit(x = X,y = y,family = binomial(),
                        weights = weights,offset = offs)
    mu_glm <- glm_temp$fitted.values
    res <- y - mu_glm
    rho <- as.numeric( t(res) %*% (W %*% res) / (t(res) %*% res) )
    rho <- max(min(rho, 0.95), -0.95)
  }else{
    rho <- 0
  }
  A <- Matrix::Diagonal(n) - rho*W
  
  beta_new <- try(glm.fit(y = y, x = Matrix::solve(A + Matrix::Diagonal(n, eps),X), family = family2, 
                          weights = weights, offset = offs), silent = TRUE)
  if(!inherits(beta_new, "try-error")){
    beta_new <- matrix(beta_new$coefficients, nrow=p)
  }else if(!inherits(modTemp, "try-error")){
    beta_new <- matrix(modTemp$coefficients[1:p], nrow=p)
  }else{
    beta_new <- matrix(0, ncol=p)
  }
  
  rho_f <- optim(par=rho,qllp, beta=beta_new, W=W, D=datas,n=n, method="L-BFGS-B", lower=-0.99,upper=0.99)
  rho_new <- rho_f$par
  tolrho <- 1
  niterrho <- 1
 
  Result <- rho_new
  while(tolrho>toler & niterrho < maxit){
    rho <- rho_new
    A = Matrix::Diagonal(n) - rho*W
    
    tol <- 1
    niter <- 1
    
    while(tol > toler & niter < maxit){
      
      beta_old <- beta_new
      
      Xi <- Matrix::solve(A + Matrix::Diagonal(n,eps), X)
      
      eta <- Xi %*% beta_old + offs
      mu  <- family$linkinv(eta[,1])
      
      gprime <- family$mu.eta(eta[,1])
      varmu  <- family$variance(mu)
      
      wgee <- weights * gprime^2 / varmu
      
      H <- Matrix::crossprod(Xi, wgee * Xi)
      
      U <- Matrix::crossprod(Xi, weights* gprime/varmu * (y - mu))
      
      beta_new <- beta_old + Matrix::solve(H, U)
      
      tol <- max(abs((beta_new - beta_old)/(beta_old + 1e-8)))
      
      niter <- niter + 1
    }
    
    rho_f <- optim(par=rho,qllp, beta=beta_new, W=W, D=datas, n=n,
                   method="L-BFGS-B", lower=-0.99,upper=0.99, hessian = TRUE)
    
    rho_new <- rho_f$par
    varrho <- -rho_f$hessian[1,1]
    tolrho <- abs(rho_new-rho)
    
    niterrho <- niterrho + 1
    Result <- c(Result, rho_new)
  }
  #print(rho_f)
  rho <- rho_new
  A <- Matrix::Diagonal(n) - rho * W
  Xi <- Matrix::solve(A + Matrix::Diagonal(n, eps), X)
  eta <- Xi %*% beta_new + offs
  mu  <- family$linkinv(eta[,1])
  
  gprime <- family$mu.eta(eta[,1])
  varmu  <- family$variance(mu)
  
  wgee <- weights * gprime^2 / varmu
  
  w <- sqrt(weights * gprime^2 / varmu)
  Xw <- sweep(Xi, 1, w, "*")
  
  phi <- sum((y - mu)^2 / (varmu / weights)) / (n - p)
  
  # Información de Fisher aproximada (H) y score (U)
  H <- Matrix::crossprod(Xi, wgee * Xi)
  U <- Matrix::crossprod(Xi, weights * gprime / varmu * (y - mu))
  Ui <- Xi * as.vector(weights * gprime / varmu * (y - mu))
  
  # Varianza sandwich directamente
  I0 <- Matrix::solve(H)
  B <- crossprod(Ui)
  vcovs <- I0 %*% B %*% I0

  # Ajuste especial para binomial con weights = 1
  if(family$family == "binomial" & all(weights == 1)){
    vcovs <- I0
  }
  
  rownames(vcovs) <- colnames(X)
  colnames(vcovs) <- colnames(X)
  
  # Cálculo de CIC
  CIC <- sum(Matrix::diag(I0 %*% B))/phi
  
  # RJC
  # Log-likelihood (quasi)
  logLik <- -qllp(rho = rho, W = W, D = datas, beta = beta_new, n = n)
  
  RJC <- 2*CIC-2*logLik/phi
  
  # Varianza de rho
  inv_var_rho <- var_rho_inv(A, W, X, beta_new, family,y, weights, phi)
  var_rho <- 1 / inv_var_rho[1]
  var_rho_san <- inv_var_rho[2]
  
  
  estfun <- Matrix::crossprod(Xw, (y - mu))
  rownames(estfun) <- colnames(X)
  colnames(estfun) <- ""
  out_ <- list(coefficients = beta_new, rho=rho, varrho=var_rho, 
               varrhoSan=var_rho_san,    fitted.values = mu, 
               linear.predictors = eta,  arrangedata = datas, vcovs=vcovs*phi,
               prior.weights = weights, y = y, formula = formula, call = match.call(), 
               offset = offs, model = mf, data = data,  
               converged = ifelse(niterrho < maxit, TRUE, FALSE), estfun = estfun, 
               R = vcovs,naive = I0*phi, family = family,  
               phi = phi,CIC = CIC, RJC = RJC, 
               logLik = logLik, deviance = sum(family$dev.resids(y, mu, weights)), 
               df.residual = length(y) - length(beta_new), 
               levels = .getXlevels(attr(mf, "terms"), mf),
               contrasts = attr(X, "contrasts"), 
               start = start, iter = niterrho, linear = TRUE)
  class(out_) <- "GSCIMC"
  return(out_)
}

#' Custom Summary Function for SARARgamlss and GSCIMC Models
#'
#' This function generates a summary for objects of class 'SARARgamlss' or 'GSCIMC'.
#' It combines the summary outputs for both models, including GAMLSS model details, 
#' spatial parameters (rho and lambda), and Wald tests.
#'
#' @param object An object of class 'SARARgamlss' or 'GSCIMC'.
#' @return A list containing the summary for the specified model class.
#' @examples
#' \donttest{
#' library(spdep)
#' library(gamlss)
#' data(oldcol)
#' W1 <- spdep::nb2mat(COL.nb, style = "W")
#' W2 <- W1  # In this case, assume the same spatial weights for both
#' # Fit a SARARgamlss model
#' result_sarar <- SARARgamlss(formula = CRIME ~ INC + HOVAL, 
#'                             sigma.formula = ~ INC + pb(HOVAL), 
#'                             W1 = W1, W2 = W2, data = COL.OLD,
#'                             type="SAR")
#' summary_SAR(result_sarar)
#' 
#' # Example for GSCIMC model
#' result_GSCIMC <- GSCIMC(formula = CRIME ~ INC + HOVAL, data = COL.OLD, W = W1)
#' summary_SAR(result_GSCIMC)
#' }
#' @export

summary_SAR <- function(object) {
  
  # Verificar si el objeto es de clase 'SARARgamlss'
  if (!is.null(object$gamlss)) {
    
    object$gamlss$call$data <- NULL
    # Resumen del modelo GAMLSS dentro del objeto SARARgamlss
    sum_gamlss <- summary(object$gamlss, type="qr")
    
    # Extraer parámetros espaciales de SARARgamlss
    rho_hat <- object$spatial$spatial[1]
    lambda_hat <- object$spatial$spatial[2]
    var_rho <- object$spatial$sdspatial[1, 1]
    var_lambda <- object$spatial$sdspatial[2, 2]
    
    # Calcular estadísticas de Wald para rho y lambda
    z_rho <- rho_hat / sqrt(var_rho)
    z_lambda <- lambda_hat / sqrt(var_lambda)
    
    spatial_summary <- data.frame(
      Estimate = c(rho_hat, lambda_hat),
      `Std. Error` = c(sqrt(var_rho), sqrt(var_lambda)),
      `Wald Statistic` = c(z_rho, z_lambda),
      `Pr(>|W|)` = 2 * pnorm(abs(c(z_rho, z_lambda)), lower.tail = FALSE)
    )
    rownames(spatial_summary) <- c("rho", "lambda")
    colnames(spatial_summary) <- c("Estimate", "Std.Error", "z value", "Pr(>|z|)")
    
    # Realizar prueba conjunta de Wald
    waldT <- matrix(c(rho_hat, lambda_hat), ncol = 2) %*% solve(object$spatial$sdspatial) %*% 
      matrix(c(rho_hat, lambda_hat), ncol = 1)
    pvalue <- pchisq(waldT, df = 2, lower.tail = FALSE)
    wald_test <- cbind(Wald = waldT, `Pr(>W)` = pvalue)
    colnames(wald_test) <- c("Wald value", "Pr(>w)")
    rownames(wald_test) <- c("c(rho, lambda)=c(0,0)")
    
    # Crear lista de resultados
    summary_obj <- list(
      gamlss_summary = sum_gamlss,
      spatial_summary = spatial_summary,
      wald_test = wald_test
    )
    
    class(summary_obj) <- "summary.SARARgamlss"
    return(summary_obj)
  }
  
  # Verificar si el objeto es de clase 'GSCIMC'
  else if (inherits(object, "GSCIMC")) {
    
    beta_hat <- object$coefficients
    vcov_matrix <- object$vcovs
    std_error <- sqrt(Matrix::diag(vcov_matrix))
    Z_stat <- as.vector(beta_hat / std_error)
    p_values <- 2 * (1 - pnorm(abs(Z_stat)))
    
    rho_hat <- object$rho
    varrho_hat <- object$varrho  
    
    coef_table <- cbind(
      Estimate = beta_hat,
      Std.Error = std_error,
      Z.value = Z_stat,
      P.value = p_values
    )
    colnames(coef_table) <- c("Estimate", "Std.Error", "z value", "P(>|z|)")
    
    summary_obj <- list(
      coefficients = coef_table,
      rho = rho_hat,
      varrho = varrho_hat,
      dispersion = object$phi,
      df.residual = object$df.residual,
      call = object$call,
      family = object$family$family,
      logLik = object$logLik,
      CIC = object$CIC,
      RJC = object$RJC
    )
    
    class(summary_obj) <- "summary.GSCIMC"
    return(summary_obj)
  }
  
  # Si el objeto no pertenece a ninguna de las clases, lanzar error
  else {
    stop("The object must be of class 'SARARgamlss' or 'GSCIMC'.")
  }
}

#' Print Method for Summary of SARARgamlss Models
#'
#' This method prints a formatted summary of a 'summary.SARARgamlss' object,
#' including details of the GAMLSS model, spatial parameters (rho and lambda), 
#' and Wald tests.
#'
#' @param x An object of class 'summary.SARARgamlss'.
#' @param ... Additional arguments (currently unused).
#' @return Print a summary for the specified GAMLSS model.
#' @export
print.summary.SARARgamlss <- function(x, ...) {
  cat("\nSpatial Parameters Summary:\n")
  print(x$spatial_summary)
  cat("---------------------------------------------------------\n")
  
  # Prueba conjunta de Wald
  if (!is.null(x$wald_test)) {
    cat("\nJoint Wald Test for Spatial Parameters (rho, lambda):\n")
    print(x$wald_test)
  }
  
  cat("=========================================================\n")
}

#' Print Method for Summary of GSCIMC Models
#'
#' This method prints a formatted summary of a 'summary.GSCIMC' object,
#' including details of the model coefficients, rho, dispersion, and other statistics.
#'
#' @param x An object of class 'summary.GSCIMC'.
#' @param ... Additional arguments (currently unused).
#' @return Print a summary for the specified Generalized Spatial Autoregresive Model class.
#' @export
print.summary.GSCIMC <- function(x, ...) {
  cat("\nSummary of GSCIMC Model\n")
  cat("=========================================================\n")
  cat("Family:", x$family, "\n")
  cat("Call:", deparse(x$call), "\n")
  cat("---------------------------------------------------------\n")
  cat("Coefficients:\n")
  printCoefmat(x$coefficients, digits = 4, signif.stars = TRUE)
  cat("---------------------------------------------------------\n")
  cat("Estimated rho:", formatC(x$rho, digits = 4, format = "f"), "\n")
  cat("Estimated variance of rho (varrho):", formatC(x$varrho, digits = 4, format = "f"), "\n")
  cat("Estimated dispersion:", formatC(x$dispersion, digits = 4, format = "f"), "\n")
  cat("Log-likelihood:", formatC(x$logLik, digits = 4, format = "f"), "\n")
  cat("CIC:", formatC(x$CIC, digits = 4, format = "f"), "\n")
  cat("RJC:", formatC(x$RJC, digits = 4, format = "f"), "\n")
  cat("Residual degrees of freedom:", x$df.residual, "\n")
  cat("=========================================================\n")
}

#' Compute the Inverse Variance of the Spatial Autoregressive Parameter (rho)
#'
#' This function calculates the inverse of the variance of the spatial autoregressive parameter 
#' \eqn{\rho} in a generalized spatial autoregressive (GSAR) or GEE-SAR model. 
#' The calculation is based on the quasi-likelihood derivatives with respect to \eqn{\rho} 
#' for different exponential family distributions.
#'
#' @param A Matrix. The spatial transformation matrix \eqn{\mathbf{A} = \mathbf{I} - \rho \mathbf{W}}, 
#' typically of class `Matrix`.
#' @param W Matrix. Row-standardized spatial weights matrix \eqn{\mathbf{W}} of dimension \eqn{n \times n}.
#' @param X Matrix. Design matrix of covariates, dimension \eqn{n \times p}.
#' @param beta Numeric vector. Current estimates of regression coefficients \eqn{\pmb{\beta}}, length \eqn{p}.
#' @param family GLM family object. The response distribution family (e.g., `gaussian()`, `poisson()`, `binomial()`, `Gamma()`, `Negative Binomial()`).
#' @param weights Numeric vector. Observation weights \eqn{m_i} (e.g., number of trials for binomial data), length \eqn{n}.
#' @param phi Numeric. Dispersion parameter, used for `gaussian`, `Gamma`, or `Negative Binomial` families. Default is 1.
#' @param offs Numeric vector. Optional offset vector, length \eqn{n}. Default is 0.
#' @param y response variable
#'
#' @return Numeric. The inverse of the variance of \eqn{\hat{\rho}} (\eqn{\operatorname{Var}(\hat{\rho})^{-1}}).
#'
#' @details
#' The function computes first and second derivatives of the mean \eqn{\mu_i} with respect to 
#' \eqn{\rho}, and then applies the appropriate formula for the inverse variance based on the 
#' selected family. This generalizes the quasi-likelihood derivations for spatially correlated 
#' generalized linear models.
#'
#' For binomial families with large \eqn{m_i}, it is recommended to truncate \eqn{\mu_i} within 
#' \eqn{[1e-10, 1-1e-10]} to avoid numerical instability.
#'
#' @examples
#' \dontrun{
#' library(Matrix)
#' n <- 10
#' W <- Matrix(0,n,n)
#' diag(W[-1,]) <- 1
#' X <- matrix(rnorm(n*2), n, 2)
#' beta <- c(0.5, -0.2)
#' rho <- 0.3
#' A <- Diagonal(n) - rho*W
#' family <- binomial()
#' weights <- rep(1,n)
#' var_rho_inv(A, W, X, beta, family, weights)
#' }
#'
#' @export

var_rho_inv <- function(A, W, X, beta, family, y, offs=NULL, weights=NULL, phi=1){
  n <- nrow(X)
  p <- ncol(X)
  Ainv <- Matrix::solve(A)
  tildeX <- Ainv %*% X
  eta <- as.vector(tildeX %*% beta + offs)
  mu <- family$linkinv(eta)
  g1 <- family$mu.eta(eta)
  deta <- Matrix::rowSums(Ainv %*% W %*% tildeX * Matrix::Matrix(as.vector(beta), ncol=p, nrow=n))
  dmu <- g1 * deta
  
  if(is.null(weights)) weights <- rep(1,n)
  
  # derivadas segundas de la quasi-likelihood
  fam <- family$family
  d2ell <- rep(0,n)
  d1ell <- rep(0,n)
  
  if(fam=="gaussian"){
    d2ell <- -weights / phi
    d1ell <- weights * (y - mu) / phi
  } else if(fam=="poisson" || fam=="ptfamily"){
    d2ell <- -weights / mu
    d1ell <- weights * (y - mu) / mu
  } else if(fam=="binomial"){
    m <- weights
    d2ell <- weights * (-m/mu^2 + (m-1)/(1-mu)^2)
    d1ell <- weights * (y - mu) / (mu*(1-mu))  # aproximación
  } else if(fam=="Gamma"){
    d2ell <- -weights * (1/mu^2 - 1/(mu*phi))
    d1ell <- weights * (y - mu)/mu^2
  } else if(fam=="Negative Binomial"){
    d2ell <- -weights / (mu + mu^2/phi)
    d1ell <- weights * (y - mu) / (mu + mu^2/phi)
  } else stop("Family not implemented")
  
  # sandwich
  J <- sum((dmu^2) * (-d2ell))  # información esperada
  U <- d1ell * dmu
  var_rho_sandwich <- (1/J) * sum(U^2) * (1/J)
  
  return(c(J/phi,phi*var_rho_sandwich))
}

#' SARARgamlss: Spatial Autoregressive Generalized Additive Model for Location Scale (GAMLSS)
#'
#' This function estimates a Spatial Autoregressive Generalized Additive Model for Location Scale 
#' (SARARgamlss) using GAMLSS. The model includes both spatial dependencies and the possibility of 
#' non-parametric terms in the formulas for the mean and variance. The function supports SAR, SARAR, 
#' and SEM model types and performs the estimation through an iterative process that updates spatial 
#' dependence parameters. The variance of the spatial parameters \eqn{\hat{\rho}} and \eqn{\hat{\lambda}}
#' is estimated using the inverse of the Hessian matrix from the optimization.
#'
#' @param formula A formula specifying the mean structure of the model (response ~ explanatory variables).
#' @param sigma.formula A formula specifying the variance structure of the model (default: ~1).
#' @param W1 A spatial weights matrix for the SAR term (default: identity matrix).
#' @param W2 A spatial weights matrix for the SARAR term (default: identity matrix).
#' @param data A data.frame containing the variables used in the model.
#' @param tol Convergence tolerance (default: 1E-4).
#' @param maxiter Maximum number of iterations for optimization (default: 20).
#' @param type The type of spatial model to fit: one of "SAR", "SARAR", or "SEM".
#' @param weights Optional weights for the observations (default: NULL).
#' 
#' @return A fitted GAMLSS model object with spatial autoregressive terms. The model object also includes
#' the variance of the spatial parameters \eqn{\hat{\rho}} and \eqn{\hat{\lambda}}
#' @references Toloza-Delgado, J. D., Melo, O. O., & Cruz, N. A.
#'  Joint spatial modeling of mean and non-homogeneous variance combining semiparametric SAR 
#'  and GAMLSS models for hedonic prices. Spatial Statistics, 65, 100864 (2025)
#'  @source https://doi.org/10.1016/j.spasta.2024.100864
#' @examples
#' library(spdep)
#' library(gamlss)
#' data(oldcol)
#' # Create spatial weight matrices W1 and W2
#' W1 <- spdep::nb2mat(COL.nb, style = "W")
#' W2 <- W1  # In this case, assume the same spatial weights for both
#' # Fit a SARARgamlss model
#' result <- SARARgamlss(formula = CRIME ~ INC + cs(HOVAL), 
#' sigma.formula = ~ INC + pb(HOVAL), W1 = W1, W2 = W2,data = COL.OLD, 
#' tol = 1E-4,  maxiter = 20, type = "SARAR")
#' summary_SAR(result)
#' gamlss::term.plot(result$gamlss, what="mu")
#' 
#' @export
#' @importFrom methods is
#' @importFrom stats .getXlevels binomial dpois fitted gaussian glm.fit 
#' @importFrom stats make.link model.matrix model.offset model.response coef model.frame
#' @importFrom stats optim pchisq pnorm predict printCoefmat rpois update
#' @import gamlss
#' @import splines
#' @importFrom gamlss.dist NO

SARARgamlss <- function(formula, sigma.formula = ~1,
                        W1 = diag(0, nrow(data)), W2 = diag(0, nrow(data)),
                        data, tol = 1E-4, maxiter = 20,
                        type = c("SAR", "SARAR", "SEM"),
                        weights = NULL) {
  mf <- stats::model.frame(formula, data=data)
  data_name <- substitute(data) 
  if (type == "SAR") {
    W2 = 0 * W2
  }
  if (type == "SEM") {
    W1 = 0 * W1
  }
  if (type == "SARAR" & sum(W2) == 0) {
    W2 <- W1
  }
  
  n <- nrow(data)
  # Fit the initial GAMLSS model for mean (mu) and variance (sigma)
  m0 <- gamlss::gamlss(formula = formula, sigma.formula = sigma.formula, 
                       data = data, family = NO())
  
  profile_loglik <- function(par, W1, W2, formula, sigma.formula, data){
    
    rho <- par[1]
    lambda <- par[2]
    
    n <- nrow(data)
    
    AA <- Matrix::Diagonal(n) - rho * W1
    BB <- Matrix::Diagonal(n) - lambda * W2
    
    Y <- model.response(model.frame(formula,data))
    Ytemp <- as.vector(BB %*% AA %*% Y)
    
    mf <- model.frame(formula,data)
    X <- model.matrix(attr(mf,"terms"),mf)
    
    Xtemp <- as.matrix(BB %*% X)
    
    datatemp <- data
    datatemp$Ytemp <- Ytemp
    
    m <- gamlss::gamlss(
      Ytemp ~ Xtemp -1,
      sigma.formula = sigma.formula,
      family = NO(),
      data = datatemp,
      trace = FALSE
    )
    
    mu <- predict(m,"mu",type="response")
    sigma2 <- predict(m,"sigma",type="response")^2
    
    detAA <- Matrix::determinant(AA,logarithm=TRUE)
    detBB <- Matrix::determinant(BB,logarithm=TRUE)
    
    if(detAA$sign <=0 | detBB$sign <=0) return(1e12)
    
    ll <- detAA$modulus + detBB$modulus -
      0.5*sum(log(sigma2)) -
      0.5*sum((Ytemp-mu)^2/sigma2)
    
    return(-ll)
  }
  
  opt <- optim( c(0,0),  profile_loglik,W1=W1,W2=W2,formula=formula,
    sigma.formula=sigma.formula,data=data,method="L-BFGS-B",
    lower=c(-0.999,-0.999), upper=c(0.999,0.999), hessian = TRUE)
  
  Y <- model.response(model.frame(formula,data))
  p0 <- opt$par
  AA <- Matrix::Diagonal(n) - p0[1] * W1
  BB <- Matrix::Diagonal(n) - p0[2] * W2
  Ytemp <- as.matrix(BB %*% AA %*% Y)
  Xtemp <- as.matrix(BB %*% model.matrix(m0, what = "mu"))
  Ztemp <- model.matrix(m0, what = "sigma")
  Hessian <- opt$hessian  # Extract Hessian matrix
  
  colnames(Xtemp) <- colnames(model.matrix(m0, what = "mu"))
  colnames(Ztemp) <- colnames(model.matrix(m0, what = "sigma"))
  
  
  # Fit updated GAMLSS model with transformed data (dependent and independent)
  m1 <- gamlss::gamlss(Ytemp ~ Xtemp - 1, ~Ztemp - 1, family = NO())
  var1 <- predict(m1, what = "sigma", type = "response")^2
  Xbeta <- Matrix::solve(BB,predict(m1, what = "mu", type = "response"))
  # Store updated model parameters
  
  names_to_update <- c("G.deviance", "residuals", "mu.fv", "mu.lp", "mu.wv", 
                       "mu.wt", "mu.qr", "mu.coefficients", "sigma.fv",
                       "sigma.wv", "sigma.wt", "sigma.coefficients", "sigma.qr",
                       "P.deviance", "aic", "sbc")
  
  for(valu in names_to_update){
    names(m1[[valu]]) <- names(m0[[valu]])
  }
  
  for (names in names_to_update) {
    m0[[names]] <- m1[[names]]
  }
  
  #model.frame(m0, what="mu") <- 
  # Return the final model object with spatial parameters and variance estimates
  if (type == "SARAR") {
    spamu <- c(p0[1],p0[2])
    var_cov_matrix <- solve(Hessian)
    spacov = var_cov_matrix
  } else if (type == "SAR") {
    spamu <- c(p0[1], NA)
    var_cov_matrix <- matrix(c( solve(Hessian[1,1]),NA, NA,NA), ncol=2, nrow=2)
    spacov = var_cov_matrix
  } else {
    spamu <- c(NA, p0[2])
    var_cov_matrix <- matrix(c(NA, NA,NA, solve(Hessian[2,2])), ncol=2, nrow=2)
    spacov =var_cov_matrix 
  }
  # model.frame(m0)[,1] <- as.vector(model.frame(m1)[,1])
  m0$call$data <- data_name
  
  X <- model.matrix(m0, what = "mu")
  beta <- stats::coef(m0, "mu")
  Omega <- Matrix::diag(predict(m0, what = "sigma")^2)
  rho <- p0[1]
  lambda <- p0[2]
  AA <- Matrix::Diagonal(n) - rho * W1
  BB <- Matrix::Diagonal(n) - lambda * W2
  BBInv <- Matrix::solve(BB)
  varU <- BBInv%*%Omega %*% Matrix::t(BBInv)
  
  inv_cov <- Matrix::solve(lambda^2 * W2 %*% varU %*% Matrix::t(W2) +
                     lambda *  Omega %*% Matrix::t(BBInv) %*% Matrix::t(W2) +
                     lambda * W2 %*% BBInv %*% Omega +
                     Omega)
  y_true <- Y
  y_signal <- (lambda* W2 %*% varU + BBInv %*% Omega) %*% inv_cov %*% 
    (AA%*%y_true -  X %*% beta)
  
  y_trend <- rho * W1 %*% y_true + X %*% beta
  y_blup <- y_trend + y_signal
  residuals <- AA%*%y_true -  X %*% beta
  y_noise <- y_true - y_blup
  
  out1 <- list(gamlss=m0, model = mf,
               spatial=list(spatial=spamu, sdspatial=spacov, type=type),
               gamlssAY=m1,  y_signal = y_signal, y_trend = y_trend,
               y_blup = y_blup, residuals = residuals,y_noise = y_noise)
  #out1$call <- match.call()
  class(out1) <- "SARARgamlss"
  return(out1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ptfamily.R
\name{ptfamily}
\alias{ptfamily}
\title{Truncated Poisson Family for GLM}
\usage{
ptfamily(link = "log")
}
\arguments{
\item{link}{Character string or a link-glm object specifying the link function. Accepted values are "log", "identity", and "sqrt".}
}
\value{
An object of class "family" that can be used in \code{glm()}.
}
\description{
This function defines a truncated Poisson family for use in Generalized Linear Models (GLMs), 
where zero values are not allowed. It modifies the Poisson likelihood by excluding zero-count observations.
}
\examples{
set.seed(123)
y <- rpois(100, lambda = 3)
y <- y[y > 0]  # Truncate zeros
x <- rnorm(length(y))
model <- glm(y ~ x, family = ptfamily())
summary(model)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSCIMC.R
\name{GSCIMC}
\alias{GSCIMC}
\title{Generalized Estimating Equations with Spatial Autoregressive Components}
\source{
https://doi.org/10.48550/arXiv.2412.00945
}
\usage{
GSCIMC(
  formula,
  family = gaussian(),
  weights = NULL,
  data,
  W,
  start = NULL,
  toler = 1e-04,
  maxit = 200,
  trace = FALSE,
  eps = 1e-06
)
}
\arguments{
\item{formula}{A formula specifying the model structure (response ~ predictors).}

\item{family}{A description of the error distribution and link function. Default is `gaussian()`.}

\item{weights}{Optional vector of prior weights. Must be positive.}

\item{data}{A data frame containing the variables in the model.}

\item{W}{A spatial weights matrix defining the spatial dependence structure.}

\item{start}{Optional starting values for parameter estimation.}

\item{toler}{Convergence tolerance for iterative optimization. Default is `1e-05`.}

\item{maxit}{Maximum number of iterations for model fitting. Default is `50`.}

\item{trace}{Logical; if `TRUE`, prints iteration details. Default is `FALSE`.}

\item{eps}{Minimun value for variance diference of zero}
}
\value{
A list of class `"GSCIMC"` containing:
\item{coefficients}{Estimated regression coefficients.}
\item{rho}{Estimated spatial autoregressive parameter.}
\item{fitted.values}{Predicted values from the model.}
\item{linear.predictors}{Linear predictor values (`X * beta`).}
\item{prior.weights}{Weights used in estimation.}
\item{y}{Observed response values.}
\item{formula}{Model formula.}
\item{call}{Function call used to fit the model.}
\item{data}{Data used in the model.}
\item{converged}{Logical indicating whether the algorithm converged.}
\item{logLik}{Quasi-log-likelihood of the fitted model.}
\item{deviance}{Residual deviance.}
\item{df.residual}{Residual degrees of freedom.}
\item{phi}{Dispersion parameter estimate.}
\item{R}{Robust Variance Estimation.}
\item{CIC}{Corrected Information Criterion.}
\item{RJC}{Robust Jackknife Correction.}
}
\description{
`GSCIMC` estimates generalized estimating equations (GEE) incorporating spatial autoregressive (SAR) components.  
It extends GEE models to account for spatial dependence in the response variable.
}
\details{
The function estimates a spatially autoregressive GEE model by iteratively updating the spatial dependence 
parameter (`rho`) and regression coefficients (`beta`). The estimation follows a quasi-likelihood approach 
using iterative weighted least squares (IWLS).  

The function supports common GLM families (`gaussian`, `binomial`, `poisson`, `Gamma`, `inverse.gaussian`) and 
their quasi-likelihood equivalents.
}
\examples{
\donttest{
library(spdep)
library(sp)
data(meuse)
sp::coordinates(meuse) <- ~x+y
W <- spdep::nb2mat(knn2nb(knearneigh(meuse, k=5)), style="W")
fit <- GSCIMC(cadmium ~ dist + elev, family=poisson(), data=meuse, W=W)
summary_SAR(fit)
}
}
\references{
Cruz, N. A., Toloza-Delgado, J. D., & Melo, O. O. (2024). 
Generalized spatial autoregressive model. arXiv preprint arXiv:2412.00945.
}
\seealso{
\code{\link{glm}}, \code{\link[gee]{gee}}, \code{\link[spdep]{spdep}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.GSCIMC.R
\name{print.summary.GSCIMC}
\alias{print.summary.GSCIMC}
\title{Print Method for Summary of GSCIMC Models}
\usage{
\method{print}{summary.GSCIMC}(x, ...)
}
\arguments{
\item{x}{An object of class 'summary.GSCIMC'.}

\item{...}{Additional arguments (currently unused).}
}
\value{
Print a summary for the specified Generalized Spatial Autoregresive Model class.
}
\description{
This method prints a formatted summary of a 'summary.GSCIMC' object,
including details of the model coefficients, rho, dispersion, and other statistics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/var_rho_inv.R
\name{var_rho_inv}
\alias{var_rho_inv}
\title{Compute the Inverse Variance of the Spatial Autoregressive Parameter (rho)}
\usage{
var_rho_inv(A, W, X, beta, family, y, offs = NULL, weights = NULL, phi = 1)
}
\arguments{
\item{A}{Matrix. The spatial transformation matrix \eqn{\mathbf{A} = \mathbf{I} - \rho \mathbf{W}}, 
typically of class `Matrix`.}

\item{W}{Matrix. Row-standardized spatial weights matrix \eqn{\mathbf{W}} of dimension \eqn{n \times n}.}

\item{X}{Matrix. Design matrix of covariates, dimension \eqn{n \times p}.}

\item{beta}{Numeric vector. Current estimates of regression coefficients \eqn{\pmb{\beta}}, length \eqn{p}.}

\item{family}{GLM family object. The response distribution family (e.g., `gaussian()`, `poisson()`, `binomial()`, `Gamma()`, `Negative Binomial()`).}

\item{y}{response variable}

\item{offs}{Numeric vector. Optional offset vector, length \eqn{n}. Default is 0.}

\item{weights}{Numeric vector. Observation weights \eqn{m_i} (e.g., number of trials for binomial data), length \eqn{n}.}

\item{phi}{Numeric. Dispersion parameter, used for `gaussian`, `Gamma`, or `Negative Binomial` families. Default is 1.}
}
\value{
Numeric. The inverse of the variance of \eqn{\hat{\rho}} (\eqn{\operatorname{Var}(\hat{\rho})^{-1}}).
}
\description{
This function calculates the inverse of the variance of the spatial autoregressive parameter 
\eqn{\rho} in a generalized spatial autoregressive (GSAR) or GEE-SAR model. 
The calculation is based on the quasi-likelihood derivatives with respect to \eqn{\rho} 
for different exponential family distributions.
}
\details{
The function computes first and second derivatives of the mean \eqn{\mu_i} with respect to 
\eqn{\rho}, and then applies the appropriate formula for the inverse variance based on the 
selected family. This generalizes the quasi-likelihood derivations for spatially correlated 
generalized linear models.

For binomial families with large \eqn{m_i}, it is recommended to truncate \eqn{\mu_i} within 
\eqn{[1e-10, 1-1e-10]} to avoid numerical instability.
}
\examples{
\dontrun{
library(Matrix)
n <- 10
W <- Matrix(0,n,n)
diag(W[-1,]) <- 1
X <- matrix(rnorm(n*2), n, 2)
beta <- c(0.5, -0.2)
rho <- 0.3
A <- Diagonal(n) - rho*W
family <- binomial()
weights <- rep(1,n)
var_rho_inv(A, W, X, beta, family, weights)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.GSCIMC.R
\name{print.summary.SARARgamlss}
\alias{print.summary.SARARgamlss}
\title{Print Method for Summary of SARARgamlss Models}
\usage{
\method{print}{summary.SARARgamlss}(x, ...)
}
\arguments{
\item{x}{An object of class 'summary.SARARgamlss'.}

\item{...}{Additional arguments (currently unused).}
}
\value{
Print a summary for the specified GAMLSS model.
}
\description{
This method prints a formatted summary of a 'summary.SARARgamlss' object,
including details of the GAMLSS model, spatial parameters (rho and lambda), 
and Wald tests.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Hurdle_GSCIMC.R
\name{Hurdle_GSCIMC}
\alias{Hurdle_GSCIMC}
\title{Hurdle Model using GSCIMC}
\usage{
Hurdle_GSCIMC(
  formula,
  data,
  W,
  weights = NULL,
  toler = 1e-05,
  maxit = 200,
  trace = FALSE
)
}
\arguments{
\item{formula}{A formula specifying the model.}

\item{data}{The dataset.}

\item{W}{The spatial weight matrix.}

\item{weights}{Optional weights.}

\item{toler}{Convergence tolerance.}

\item{maxit}{Maximum number of iterations.}

\item{trace}{Logical. If TRUE, prints progress.}
}
\value{
A list containing the logit and Poisson-truncated models.
}
\description{
This function fits a hurdle model using GSCIMC, consisting of:
(1) A logit model for zero vs. non-zero responses.
(2) A truncated Poisson model for positive counts.
}
\examples{
\donttest{
set.seed(123)
n <- 100
x <- rnorm(n)
y <- rpois(n, lambda = exp(0.5 * x))
y[rbinom(n, 1, 1/(1+exp(-0.5*x)))] <- 0  # Introduce zeros
W <- matrix(rbinom(n^2,1,0.2), n, n)  # Example spatial weight matrix
diag(W) <- 0
rtot <- rowSums(W)
W <- W/ifelse(rtot==0, 0.1, rtot)
model <- Hurdle_GSCIMC(y ~ x, data = data.frame(y, x), W = W)
summary_SAR(model$logit_model)
summary_SAR(model$poisson_truncated_model)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SARARgamlss.R
\name{SARARgamlss}
\alias{SARARgamlss}
\title{SARARgamlss: Spatial Autoregressive Generalized Additive Model for Location Scale (GAMLSS)}
\usage{
SARARgamlss(
  formula,
  sigma.formula = ~1,
  W1 = diag(0, nrow(data)),
  W2 = diag(0, nrow(data)),
  data,
  tol = 1e-04,
  maxiter = 20,
  type = c("SAR", "SARAR", "SEM"),
  weights = NULL
)
}
\arguments{
\item{formula}{A formula specifying the mean structure of the model (response ~ explanatory variables).}

\item{sigma.formula}{A formula specifying the variance structure of the model (default: ~1).}

\item{W1}{A spatial weights matrix for the SAR term (default: identity matrix).}

\item{W2}{A spatial weights matrix for the SARAR term (default: identity matrix).}

\item{data}{A data.frame containing the variables used in the model.}

\item{tol}{Convergence tolerance (default: 1E-4).}

\item{maxiter}{Maximum number of iterations for optimization (default: 20).}

\item{type}{The type of spatial model to fit: one of "SAR", "SARAR", or "SEM".}

\item{weights}{Optional weights for the observations (default: NULL).}
}
\value{
A fitted GAMLSS model object with spatial autoregressive terms. The model object also includes
the variance of the spatial parameters \eqn{\hat{\rho}} and \eqn{\hat{\lambda}}
}
\description{
This function estimates a Spatial Autoregressive Generalized Additive Model for Location Scale 
(SARARgamlss) using GAMLSS. The model includes both spatial dependencies and the possibility of 
non-parametric terms in the formulas for the mean and variance. The function supports SAR, SARAR, 
and SEM model types and performs the estimation through an iterative process that updates spatial 
dependence parameters. The variance of the spatial parameters \eqn{\hat{\rho}} and \eqn{\hat{\lambda}}
is estimated using the inverse of the Hessian matrix from the optimization.
}
\examples{
library(spdep)
library(gamlss)
data(oldcol)
# Create spatial weight matrices W1 and W2
W1 <- spdep::nb2mat(COL.nb, style = "W")
W2 <- W1  # In this case, assume the same spatial weights for both
# Fit a SARARgamlss model
result <- SARARgamlss(formula = CRIME ~ INC + cs(HOVAL), 
sigma.formula = ~ INC + pb(HOVAL), W1 = W1, W2 = W2,data = COL.OLD, 
tol = 1E-4,  maxiter = 20, type = "SARAR")
summary_SAR(result)
gamlss::term.plot(result$gamlss, what="mu")

}
\references{
Toloza-Delgado, J. D., Melo, O. O., & Cruz, N. A.
 Joint spatial modeling of mean and non-homogeneous variance combining semiparametric SAR 
 and GAMLSS models for hedonic prices. Spatial Statistics, 65, 100864 (2025)
 @source https://doi.org/10.1016/j.spasta.2024.100864
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.GSCIMC.R
\name{summary_SAR}
\alias{summary_SAR}
\title{Custom Summary Function for SARARgamlss and GSCIMC Models}
\usage{
summary_SAR(object)
}
\arguments{
\item{object}{An object of class 'SARARgamlss' or 'GSCIMC'.}
}
\value{
A list containing the summary for the specified model class.
}
\description{
This function generates a summary for objects of class 'SARARgamlss' or 'GSCIMC'.
It combines the summary outputs for both models, including GAMLSS model details, 
spatial parameters (rho and lambda), and Wald tests.
}
\examples{
\donttest{
library(spdep)
library(gamlss)
data(oldcol)
W1 <- spdep::nb2mat(COL.nb, style = "W")
W2 <- W1  # In this case, assume the same spatial weights for both
# Fit a SARARgamlss model
result_sarar <- SARARgamlss(formula = CRIME ~ INC + HOVAL, 
                            sigma.formula = ~ INC + pb(HOVAL), 
                            W1 = W1, W2 = W2, data = COL.OLD,
                            type="SAR")
summary_SAR(result_sarar)

# Example for GSCIMC model
result_GSCIMC <- GSCIMC(formula = CRIME ~ INC + HOVAL, data = COL.OLD, W = W1)
summary_SAR(result_GSCIMC)
}
}
